// Algo358LF.c : Defines the entry point for the console application.

//				Tests the CSVD routine, ACM Algorithm 358
//
//		Singular Value Decomposition, a = u * s * Conj(Tran(v)), a is destroyed by CSVD
//		the diagonal matrix s is output as a vector, m must be >= n
//		if smaller, a should be filled with zero rows
//		this code is adapted from Collected Algorithms from ACM, Algorithm 358
//		The transformation Conj(Tran(u)) is applied to the p vectors given in columns
//		n, n+1, ..., n+p-1 of matrix a
//		See: http://www.scs.fsu.edu/~burkardt/f77_src/toms358/toms358.f
//		and: http://www.scs.fsu.edu/~burkardt/f77_src/toms358/toms358_prb.f

#include <stdlib.h>
#include "Algo358.h"
#include <time.h>

void CSVD(float _Complex a[][maxA], int m, int n, int p, int nu, int nv,
		float s[], float _Complex u[][maxA], float _Complex v[][maxA]);
void TestCSVD();
void TimeStamp();
void WriteMatrix(float _Complex [][maxA], int, int);

int ti;		// test case number, 1 or 2
double time0;

// ------- initialize
void init(void)
{
	char name[]    = "Algo358LF - Tests the CSVD routine, ACM Algorithm 358";
	char version[] = "Lcc-Win32 Console Version 1.1.1";
	char author1[] = "Copyright (c) 1981-2007 by author: Harry J. Smith,";
	char author2[] = "19628 Via Monte Dr., Saratoga, CA 95070.  All rights reserved.";
	char usage1[]  = "Usage: Algo358LF [1, 2] [>Test.out]";
	printf("\n%s\n", name);		// show credits
	printf("%s\n", version);
	printf("%s\n", author1);
	printf("%s\n\n", author2);
	printf("%s\n\n", usage1);	// show usage
} // init

// The main entry point for the application Algo358LF
//int _tmain(int argc, _TCHAR* argv[])
int main(int argc, char* argv[])
{
	init();
	char tc[5] = "1";
	if (argc > 1)							// if arguments
		strncpy(tc, argv[1], 2);		// Copy string from user input line
	ti = atoi(tc);
	if (ti != 1 && ti != 2)
		ti = 1;
	printf("Running test case %i of 2\n\n", ti);
	time0 = (double)clock() / (double)CLOCKS_PER_SEC;
	TimeStamp();
	printf("\nAlgo358LF:\n");
	printf("Test the ACM TOMS 358 Algorithm CSVD\n\n");
	TestCSVD();
	printf("Algo358LF:\n");
	printf("Normal end of execution\n\n");
	TimeStamp();
	return 0;
 	} // main

float _Complex SetTo(float x, float y)		// Set Real and Imag parts
{
	return x + I * y;
} // SetTo

void TestCSVD()		//test CSVD
{
	int i, j, k;
	int m = 5;
	int n = 5;
	if (ti == 2)
		n = 4;
	float _Complex a[maxA][maxA];
	float s[maxA];
	float _Complex u[maxA][maxA];
	float _Complex v[maxA][maxA];
	printf("TestCSVD\n");
	printf("Call ACM TOMS Algorithm 358 for the\n");
	printf("singular value decomposition:\n");
	printf("  A = U * S * V^H\n");
	printf("  of an M by N complex matrix A.\n");
	float _Complex cZero = SetTo(0.0F, 0.0F);
	a[0][0] = SetTo( 0.4499F, -0.1267F);	// Set the matrix
	a[0][1] = SetTo(-0.2361F,  0.0775F);
	a[0][2] = SetTo( 0.5008F, -0.7799F);
	a[0][3] = SetTo(-0.7702F, -0.3143F);
	a[0][4] = SetTo(-0.5084F,  0.5621F);
	a[1][0] = SetTo(-0.8432F, -0.3443F);
	a[1][1] = SetTo( 0.0186F, -0.6332F);
	a[1][2] = SetTo( 0.3505F,  0.0166F);
	a[1][3] = SetTo(-0.8892F,  0.2657F);
	a[1][4] = SetTo(-0.5066F,  0.6005F);
	a[2][0] = SetTo( 0.5896F,  0.2601F);
	a[2][1] = SetTo( 0.8928F,  0.0103F);
	a[2][2] = SetTo( 0.4350F, -0.2666F);
	a[2][3] = SetTo(-0.7799F, -0.5512F);
	a[2][4] = SetTo(-0.1022F, -0.4500F);
	a[3][0] = SetTo( 0.3911F,  0.3234F);
	a[3][1] = SetTo(-0.5605F,  0.7638F);
	a[3][2] = SetTo(-0.2009F,  0.2707F);
	a[3][3] = SetTo( 0.0314F, -0.4336F);
	a[3][4] = SetTo(-0.1046F,  0.3267F);
	a[4][0] = SetTo(-0.1395F, -0.1561F);
	a[4][1] = SetTo( 0.3064F,  0.0263F);
	a[4][2] = SetTo(-0.0975F,  0.9019F);
	a[4][3] = SetTo( 0.2491F,  0.5787F);
	a[4][4] = SetTo( 0.4106F, -0.8102F);
	printf("\nMatrix A:\n\n");
	WriteMatrix(a, m, n);
	int p = 0;
	int nu = m;
	int nv = n;
	CSVD(a, m, n, p, nu, nv, s, u, v);
	printf("Singular values (Diagonal of diagonal S matrix):\n\n");
	for (i = 0; i < n; i++)
		 printf("  %.7f\n", s[i]);
	if (m > n)
		printf("\nU (Augmented to an orthogonal basis for the output space):\n\n");
	else
		printf("\nU:\n\n");
	WriteMatrix(u, m, m);
	printf("V:\n\n");
	WriteMatrix(v, n, n);
	for (i = 0; i < m; i++)		// Compute U * S * V^H
		 for (j = 0; j < n; j++)
		 {
				 a[i][j] = cZero;
				 for (k = 0; k < m && k < n; k++)
						 a[i][j] = a[i][j] + u[i][k] * s[k] * conj(v[j][k]);
		 }
	printf("Matrix U * S * V^H (Should equal the original A):\n\n");
	WriteMatrix(a, m, n);
} // TestCSVD

void WriteMatrix(float _Complex a[][maxA], int m, int n)
// print an m by n complex matrix
{
	int i, j;
	for (i = 0; i < m; i++)
	{
		for (j = 0; j < n; j++)
		{
			printf(" ");
			if (crealf(a[i][j]) >= 0.0F)
				printf(" ");
			else
				printf("-");
			if (crealf(a[i][j]) == 0.0F)
				printf("0.0000");
			else
				printf("%.4f", fabsf(crealf(a[i][j])));
			if (cimagf(a[i][j]) >= 0.0F)
				printf("+");
			else
				printf("-");
			if (cimagf(a[i][j]) == 0.0F)
				printf("0.0000");
			else
				printf("%.4f", fabsf(cimagf(a[i][j])));
		}
		printf("\n");
	}
	printf("\n");
} // WriteMatrix

void TimeStamp()
// printout the current MM/dd/yyyy hh:mm:ss AM/PM  s.ss sec. as a timestamp
{
	int i;
	double time1 = (double)clock() / (double)CLOCKS_PER_SEC;
	double timeD = time1 - time0;
	time0 = time1;
	char date[9];
	struct tm *newtime;
	time_t ltime;
	/* get time in seconds */
	time(&ltime);
	/* convert it to the structure tm */
	newtime = localtime(&ltime);
	char sta[30];
	for (i = 0; i < 24; i++)
	{
		sta[i] = (char)asctime(newtime)[i];
	}
	sta[24] = 0;
	printf("%s  %.2f sec.\n", sta, timeD);
} // TimeStamp

/* end of file Algo385LF.c ------------------------------------------ */
